@@ -232,10 +232,7 @@ def test_invalid_routes():
     """
     Test URLRouter route validation
     """
-    try:
-        from django.urls import include
-    except ImportError:  # Django 1.11
-        from django.conf.urls import include
+    from django.urls import include
 
     with pytest.raises(ImproperlyConfigured) as exc:
         URLRouter([url(r"^$", include([]))])
